@@ -5,6 +5,8 @@ from .alps_cthyb import ALPSCTHYBSolver
 from .null_solver import NullSolver
 from .alps_cthyb_seg import ALPSCTHYBSEGSolver
 from .pomerol import PomerolSolver
+from .hphi import HPhiSolver
+from .jo_cthyb_seg import JOCTHYBSEGSolver
 from .base import compute_basis_rot
 
 solver_classes = {
@@ -15,4 +17,6 @@ solver_classes = {
     'null': NullSolver,
     'ALPS/cthyb-seg': ALPSCTHYBSEGSolver,
     'pomerol': PomerolSolver,
+    'HPhi': HPhiSolver,
+    'JO/cthyb-seg': JOCTHYBSEGSolver,
 }
